/*
 *
 * Name:        io.c
 *
 * Description: input/output routines for video/audio data
 *
 * Author:	Janusz Bulawa, NPAC
 *
 * Date:	july 1996
 */

#include <stdio.h>
#include <strings.h>
#include <fcntl.h>

#define LOAD	1024

/* prototypes */

int IOopenVideoFile(char *filename);
FILE *IOopenVideoCaptionFile(char *filename);
int IOopenAudioFile(char *filename);
int IOreadNextVideoFrame(int fd, int framenum, char* buffer, int *search);
int IOreadNextAudioChunk(int fd, char* buffer, int num);
void IOcloseVideoFile(int fd);
void IOcloseVideoCaptionFile(FILE *fd);
void IOcloseAudioFile(int fd);

extern moviecapfd;
int tempfd;
int left;
int lastresult;

/* definitions */

int IOopenVideoFile(char *filename)
{
  int result;

  if ((result=open(filename, O_RDWR))<0) printf("Error opening Video file!\n");
  moviecapfd=IOopenVideoCaptionFile(filename);
  if (!moviecapfd)
    {
      printf("Caption File open failure!!!\n");
      return -1;
    }
  
  printf("Video caption file ID=%d\n", moviecapfd);
/*  tempfd=open("/home/R11A/bulawa/temp.263", O_RDWR|O_CREAT);*/
  return result;
}

FILE *IOopenVideoCaptionFile(char *filename)
{
  char *newfilename;
  FILE *result;
  
  newfilename=filename;
  strcat(newfilename, ".cap");
  printf("Caption file: %s\n", newfilename);
  if (!(result=fopen(newfilename, "r"))) printf("Error opening Caption file!\n");
  return result;
}

int IOopenAudioFile(char *filename)
{
  int result;

  if ((result=open(filename, O_RDONLY))<0) printf("Error opening Audio file!\n");
  return result;
}

int IOreadNextVideoFrame(int fd, int framenum, char* buffer, int *search)
{
  int result,k,i;
  int fnum, foffset;
  int temp;
  char *ptr;
  char done;

  if (framenum>590) return -1;

    printf("FRAME: %d search:%d\n", framenum, search);
  
  if ((*search)!=0)
  {
    printf("position in caption file:%d\n",fseek(moviecapfd, 0, 0));
    do
    {     
      fscanf(moviecapfd, "%d\t%d\n", &fnum, &foffset);
    } while (fnum!=framenum);
    printf("fnum=%d offset=%d\n", fnum, foffset);
    lseek(fd, foffset, 0);
    *search=0;
    left=0;
    lastresult=0;
  }
  
  if (left>0)
  {
    memcpy(buffer, buffer+lastresult, left);
  }

  done=0;
  result=left;
  ptr=buffer;
  k=0;
  while (!done)
  {
    for (i=4;i<left;i++)
    {
      temp=((ptr[i]<<24)+(ptr[i+1]<<16)+(ptr[i+2]<<8)+ptr[i+3])>>10;
      if (temp==32)
	{
	  done=1;
	  break; /* breaks 'for' loop */
	}
    }
    ptr+=left;
    k++;
    if (k>5) done=1;
    if (!done)
    {
/*      printf("Loading %d portion of data\n",k);*/
      left=read(fd, ptr, LOAD);
      result += left;
      if (left<=0)
        break;
    }
  }

/*  printf("left should be LOAD: %d\n", left);*/
  left=left-i;
/*  printf("left after new frame:%d\n", left);*/
  result-=left;
  lastresult=result;
/*  for (i=0;i<4;i++) printf("%d ", buffer[result+i]);
  printf("\n");

    write(tempfd, buffer, result);*/

/*  printf("result: %d left: %d \n", result, left);*/
  return result;  
}

int IOreadNextAudioChunk(int fd, char* buffer, int num)
{
  int result;
  
  result = read(fd, buffer, num);
  return result;
}

void IOcloseVideoFile(int fd)
{
/*  close(tempfd);*/
  close(fd);
  IOcloseVideoCaptionFile(moviecapfd);
}

void IOcloseVideoCaptionFile(FILE *fd)
{
  fclose(fd);
}

void IOcloseAudioFile(int fd)
{
  close(fd);
}
    





